
/* ------------------------------------------------------------------------- */

/*

ptcl.c

United States Naval Observatory
USNO/NRL Optical Interferometer 
3450 Massachusetts Avenue, NW
Washington, DC  20392-5420

Subsystem:
----------
ptcl

Description:
------------
This is the central file of the ptcl package that registers PGPLOT functions as
tcl commands.  Within it are an int main( ) function (linked from the tcl or tk
libraries) and an initialization function (which initializes tcl, tk, and
[incr tcl], registers additional tcl, tk, and [incr tcl] commands, and starts
the ptcl.tcl tcl script).

Note that the -ltk, -ltcl, and -litcl flags must be specified (IN THAT ORDER) 
in the Makefile to link tk, if desired.

Functions:
----------
main (linked from the tcl or tk object libraries), Tcl_AppInit.

Modification history:
---------------------
1995 Nov 15 - Nicholas Elias, USNO/AD5
              File created.
1995 Dec 06 - Nicholas Elias, USNO/AD5
              GETENV_NULL macro added because of Linux version of getenv( ).
1996 Jul 22 - Nicholas Elias, USNO/AD5
              TCL_VER macro added for tcl versions later than 7.3.
1996 Nov 21 - Nicholas Elias, USNO/AD5
              GETENV_NULL kludge removed.

*/

/* ------------------------------------------------------------------------- */

/* Compile check */

#if !defined TCL && !defined TK && !defined ITCL && !defined ITK
#error "At least one of the TCL, TK, ITCL, or ITK compiler directives must be set..."
#endif

/* ------------------------------------------------------------------------- */

/* Include files */

#include <stdio.h>                 /* Standard I/O */
#include <stdlib.h>                /* Standard library */
#include <math.h>                  /* Math */
#include <string.h>                /* String */
#include <malloc.h>                /* Memory allocation */
#include <signal.h>                /* Signal */

#if defined TCL && !defined TK
#include "tcl.h"                   /* tcl */
#endif

#ifdef TK
#include "tk.h"                    /* tk (already includes tcl.h) */
#endif

#ifdef ITCL
#include "itcl.h"                  /* [incr tcl] */
#endif

#ifdef ITK
#include "itk.h"                   /* [incr tk] */
#endif

#include "macro.h"                 /* Math macros */
#include "macro_tclwrap.h"         /* Math macro tcl wrapper function
                                      prototypes */
#include "message.h"               /* Message function prototypes */

#include "ptcl.h"                  /* ptcl */
#include "pgplotpackage_tclwrap.h" /* PGPLOT tcl wrapper function prototypes */

/* ------------------------------------------------------------------------- */

/* This should not have to be modified */

#ifdef __cplusplus
extern "C" {
#endif

extern int Itcl_Init _ANSI_ARGS_( (Tcl_Interp *pmtInterp ) );

/* The following is necessary to link shared libraries on Sun machines */

#ifdef Sun
extern int matherr _ANSI_ARGS_( (void) );
static int (*dummyMathPtr) _ANSI_ARGS_( (void) ) = matherr;
#endif

#ifdef __cplusplus
}
#endif

/* ------------------------------------------------------------------------- */

/*

main

Description:
------------
This is the main program.  Its form depends on the tcl version used.

Inputs:
-------
iArgC    - This is the int variable which contains the number of arguments.
**asArgV - This is the pointer to the 2D char array which contains the argument
	   list.

Outputs:
--------
None.

Modification history:
---------------------
1996 Oct 04 - Nicholas Elias, USNO/AD5
	      Function modified to handle tcl 7.3 and 7.5.

*/

/* ------------------------------------------------------------------------- */

#if TCL_VER == 730

extern int main( );                  /* This is the int main( ) function in
					tclMain.c or tkMain.c */

int *tclDummyMainPtr = (int *) main; /* This is the pointer to an int variable
                                        which allows applications to be linked
                                        with the int main function in the tcl or
                                        tk object library */

#endif

#if TCL_VER == 740 || TCL_VER == 750 || TCL_VER == 760 || TCL_VER == 800

int main( int iArgC, char **asArgV )
{

  /* Initialize tcl and/or tk */

#ifdef TK
  Tk_Main( iArgC, asArgV, Tcl_AppInit );
#else
  Tcl_Main( iArgC, asArgV, Tcl_AppInit );
#endif


  /* Return 0 to the calling function */

  return( 0 );

}

#endif

/* ------------------------------------------------------------------------- */

/*

Tcl_AppInit

Description:
------------
This procedure is called by the main program in tclMain.c or tkMain.c, which are
in the tcl or tk object libraries (it is linked via the above hack). It performs
application-specific initialization, such as initializing the interpreter, 
registering new tcl/tk commands, and starting up the ptcl.tcl script.  If this
function is successful, TCL_OK is returned, otherwise TCL_ERROR is returned.

Inputs:
-------
*pmtInterp - This is the pointer to the Tcl_Interp structure which contains
             the tcl interpreter information.

Outputs:
--------
*pmtInterp - This is the pointer to the Tcl_Interp structure which contains
             the tcl interpreter information.

Modification history:
---------------------
1995 Nov 15 - Nicholas Elias, USNO/AD5
              Function created.
1995 Dec 06 - Nicholas Elias, USNO/AD5
              GETENV_NULL macro added because of Linux version of getenv( ).
1996 Jul 22 - Nicholas Elias, USNO/AD5
              TCL_VER macro added for tcl versions later than 7.3.
1996 Nov 21 - Nicholas Elias, USNO/AD5
              GETENV_NULL kludge removed.

*/

/* ------------------------------------------------------------------------- */

int Tcl_AppInit( Tcl_Interp *pmtInterp )

{

  /* Declare the local variables */

#if TCL_VER == 740 || TCL_VER == 750 || TCL_VER == 760 || TCL_VER == 800
  char *tcl_RcFileName; /* This is the 1D char array which contains the name of
                           the ptcl start-up script */
#endif


  /* Disable ^C */

  signal( SIGINT, SIG_IGN );


  /* Start up the main window and check to see if it was successful */

#if TCL_VER == 730 && defined TK
  if ( Tk_MainWindow( pmtInterp ) == NULL ) 
  {
    return( TCL_ERROR );
  }
#endif


  /* Call the Mod_Init( pmtInterp ) procedures for the included modules;
     Tcl_Init( pmtInterp ), Tk_Init( pmtInterp ), and Itcl_Init( pmtInterp ) 
     are required.  If the calls are unsuccessful, return TCL_ERROR. */

#ifdef TCL
  if ( Tcl_Init( pmtInterp ) == TCL_ERROR ) 
  {
    return( TCL_ERROR );
  }
#endif

#ifdef TK
  if ( Tk_Init( pmtInterp ) == TCL_ERROR ) 
  {
    return( TCL_ERROR );
  }
#endif

#ifdef ITCL
  if ( Itcl_Init( pmtInterp ) == TCL_ERROR )
  {
    return( TCL_ERROR );
  }
#endif

#ifdef ITK
  if ( Itk_Init( pmtInterp ) == TCL_ERROR )
  {
    return( TCL_ERROR );
  }
#endif

  
  /* Create additional tcl commands */
 
  Macro_TclWrappers( pmtInterp ); 
  PGPLOT_TclWrappers( pmtInterp );


  /* Check to see if the ptcl environmental variable exists */

  if ( getenv( "ptcl" ) == NULL )
  {
    NrError( "!! ptcl: The \"ptcl\" environmental variable must be set..." );
  }


  /* Initialize the start-up script variable */

  tcl_RcFileName = (char*) malloc( (unsigned int) 200 * sizeof( char ) );
  sprintf( tcl_RcFileName, "%s", getenv( "ptcl" ) );
  strcat( tcl_RcFileName, "/ptcl.tcl" );

#if TCL_VER == 740 || TCL_VER == 750 || TCL_VER == 760 || TCL_VER == 800
  Tcl_SetVar( pmtInterp, "tcl_rcFileName", tcl_RcFileName, TCL_GLOBAL_ONLY );
#endif
  

  /* Return TCL_OK to the calling function */
  
  return( TCL_OK );
  
}

